#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024-06-04 15:21
# @Author  : Jack
# @File    : test_LabelStudio_UIEX

"""
test_LabelStudio_UIEX
"""

from NStudyPy import PyLabelStudio, PyFile, PyEnv, PyString


def get_train_UIEX_cert_stats():
    """
    get stats of label studio data
    """
    file_paths = PyFile.get_file_list(r'E:\Projects\pp-uiex-certs\anns\train')

    # 医师执业证
    print("#######################医师执业证###############################")
    classify_file = list(filter(lambda x: int(x.rsplit('-', 8)[1]) in [3, 4, 5], file_paths))
    PyLabelStudio.stat_UIEX_cert(file_paths=classify_file)
    # 医师资格证
    print("#######################医师资格证###############################")
    classify_file = list(filter(lambda x: int(x.rsplit('-', 8)[1]) in [7, 8, 9], file_paths))
    PyLabelStudio.stat_UIEX_cert(file_paths=classify_file)
    # 职称证
    print("#######################职称证###############################")
    classify_file = list(filter(lambda x: int(x.rsplit('-', 8)[1]) in [11, 12, 13], file_paths))
    PyLabelStudio.stat_UIEX_cert(file_paths=classify_file)

    # 合计结果
    print("#######################合计结果###############################")
    PyLabelStudio.stat_UIEX_cert(file_paths=file_paths)


def get_test_UIEX_cert_stats():
    """
    get stats of label studio data
    """
    file_paths = PyFile.get_file_list(r'E:\Projects\pp-uiex-certs\anns\test')
    # 医师执业证
    print("#######################医师执业证###############################")
    classify_file = list(filter(lambda x: int(x.rsplit('-', 8)[1]) in [6], file_paths))
    PyLabelStudio.stat_UIEX_cert(file_paths=classify_file)
    # 医师资格证
    print("#######################医师资格证###############################")
    classify_file = list(filter(lambda x: int(x.rsplit('-', 8)[1]) in [10], file_paths))
    PyLabelStudio.stat_UIEX_cert(file_paths=classify_file)
    # 职称证
    print("#######################职称证###############################")
    classify_file = list(filter(lambda x: int(x.rsplit('-', 8)[1]) in [14], file_paths))
    PyLabelStudio.stat_UIEX_cert(file_paths=classify_file)

    # 合计结果
    print("#######################合计结果###############################")
    PyLabelStudio.stat_UIEX_cert(file_paths=file_paths)


def generate_UIEX_cert_train_data():
    """
    generate train data from label studio data
    """
    file_paths = PyFile.get_file_list(r'E:\Projects\pp-uiex-certs\anns\train')
    classify_file = list(filter(lambda x: int(x.rsplit('-', 8)[1]) in [3, 4, 5], file_paths))
    PyLabelStudio.convert_to_UIEX(file_paths=classify_file, main_dir=r'E:\Projects\pp-uiex-certs\data_license', domain=PyEnv.get_env('ls_domain'), auth=PyEnv.get_env('ls_auth'))


def generate_UIEX_cert_test_data():
    """
    generate train data from label studio data
    """
    file_paths = PyFile.get_file_list(r'E:\Projects\pp-uiex-certs\anns\test')
    PyLabelStudio.convert_to_UIEX(file_paths=file_paths, main_dir=r'E:\Projects\pp-uiex-certs\data_test', domain=PyEnv.get_env('ls_domain'), auth=PyEnv.get_env('ls_auth'))


def stat_UIEX_cert_acc():
    """
    统计标签准确率
    stat_map = {
        'label': {
            'tag_count': 10,
            'pred_count': 4
        }
    }
    """
    cert_type_map = PyLabelStudio.get_UIEX_cert_type(PyFile.get_file_list(r'E:\Projects\pp-uiex-certs\anns\test'))

    stat_map = {}
    tag = '1'
    # E:\Projects\pp-uiex-certs\anns\pred
    # E:\Projects\mmdet\data\anns\pred
    # E:\Projects\mmdet\data\anns-glm-4v\pred
    dir_prefix = r'E:\Projects\mmdet\data\anns-glm-4v\pred'
    tag_map = PyFile.read_json(rf'{dir_prefix}\tag_{tag}.json')
    pred_map = PyFile.read_json(rf'{dir_prefix}\pred_{tag}.json')
    for image_id, m in tag_map.items():
        for label, v in m.items():
            label_key = f'{cert_type_map[image_id]}_{label}'
            label_v = stat_map.get(label_key, {
                'tag_count': 0,
                'pred_count': 0
            })
            pred_v = pred_map.get(image_id, {}).get(label, '')
            pred_v = label_amend(v.strip(), label, pred_v).strip()
            if v.strip().replace(' ', '') == pred_v:
                label_v['pred_count'] += 1
            else:
                print(f'{image_id:20} \t {PyString.pad_string(label, 10)}\t\t{cert_type_map[image_id]}\n'
                      f'\t\t打标值: {v}\n'
                      f'\t\t预测值: {pred_v}')
            label_v['tag_count'] += 1

            stat_map.update({
                label_key: label_v
            })

    print(f'{"版本":5}\t{"标签":10}\t{"总标签个数":5}\t{"预测对个数":5}\t{"准确率":10}')
    sum_tag = 0
    sum_pred = 0
    for label_key, v in sorted(stat_map.items(), key=lambda item: item[0]):
        cert_type, label = label_key.split('_')
        sum_tag += v['tag_count']
        sum_pred += v['pred_count']
        print(f"{cert_type:5}\t{PyString.pad_string(label, 10)}\t{v['tag_count']:5}\t{v['pred_count']:10}\t{v['pred_count'] / v['tag_count']:>10.4f}")
    print(f"{'合计':5}\t{'ALL':10}\t{sum_tag:5}\t{sum_pred:10}\t{sum_pred / sum_tag:>10.4f}")
    # for i in _temp:
    #    print(i)


import re

_temp = []
_reg_date = re.compile(r'(\d{4}).*?(\d{2}).*?(\d{2})')
_id_card = re.compile(r'(^\d{15}$)|(^\d{17}[\dXx]$)')


def format_date(date_string):
    match = _reg_date.search(date_string)
    if match:
        year, month, day = match.groups()
        return f"{year}年{month}月{day}日"
    else:
        return ""


def format_id_card(id_card_string):
    match = _id_card.search(id_card_string)
    if match:
        old_id_card, id_card = match.groups()
        return id_card or old_id_card
    else:
        return ""


def label_amend(tag, label, pred):
    pred = pred.replace(label, '').strip()
    if tag == pred:
        return pred
    if "：" in pred:
        pred = pred.rsplit("：", 1)[-1]

    if label in ['签发日期', '发证日期']:
        return format_date(pred)

    if label in ['证书编码']:
        return (pred.replace('证书', '').replace('编', '').replace("码", "")
                .replace('编号', '').replace('管理号', '').replace('资格号', ''))

    if label in ['姓名']:
        pred = pred.replace('Full Name', '')
        pred = re.sub(r'[#，]', '', pred)
        if pred.startswith('姓'):
            pred = pred[1:]
        if pred.startswith('名'):
            pred = pred[1:]
        if pred.endswith('姓'):
            pred = pred[:-1]
        return pred

    if label in ['专业']:
        pred = pred.replace('Category', '').replace('名称', '').replace('从事', '')
        return pred
    if label in ['学历']:
        if '中专' in pred:
            pred = '中专'
        elif '大专' in pred:
            pred = '大专'
        elif '本科' in pred:
            pred = '本科'
        elif '研究生' in pred:
            pred = '研究生'
        return pred

    if label in ['身份证号']:
        return format_id_card(pred)

    if label in ['资格名称/职称']:
        if '中级' in pred or '中' == pred:
            return '中级'

        pred = (pred.replace('卫生技术人员', '').replace('资格名称', '')
                .replace('职务名称', '').replace('评审通过', '').replace('合格', '')
                .replace('现有职称', '').replace('职务名称', ''))
        return pred

    if label in ['主要执业机构']:
        if pred.startswith('主要'):
            pred = pred[2:]
        if pred.endswith('执业机构'):
            pred = pred[:-4]
        return pred

    if label in ['备案机关']:
        pred = pred.replace('签发机关', '').replace('发证机关', '')
        return pred

    return pred


def gen_UIEX_cert_tag_data():
    """
    生产测试数据标签数据
    """
    file_paths = PyFile.get_file_list(r'E:\Projects\pp-uiex-certs\anns\test')
    # 医师执业证
    print("#######################医师执业证###############################")
    result = print_UIEX_cert_acc(tag=[6], file_paths=file_paths)
    PyFile.save_json(result, r'E:\Projects\pp-uiex-certs\anns\pred\tag_1.json')
    # 医师资格证
    print("#######################医师资格证###############################")
    result = print_UIEX_cert_acc(tag=[10], file_paths=file_paths)
    PyFile.save_json(result, r'E:\Projects\pp-uiex-certs\anns\pred\tag_2.json')
    # 职称证
    print("#######################职称证###############################")
    result = print_UIEX_cert_acc(tag=[14], file_paths=file_paths)
    PyFile.save_json(result, r'E:\Projects\pp-uiex-certs\anns\pred\tag_3.json')
    # 合计
    print("#######################合计###############################")
    result = print_UIEX_cert_acc(file_paths=file_paths)
    PyFile.save_json(result, r'E:\Projects\pp-uiex-certs\anns\pred\tag_all.json')


def print_UIEX_cert_acc(tag=[6, 10, 14], file_paths=[]):
    classify_file = list(filter(lambda x: int(x.rsplit('-', 8)[1]) in tag, file_paths))
    map_stat = PyLabelStudio.stat_UIEX_cert_acc(file_paths=classify_file)

    result_map = {}

    for k, v in map_stat.items():
        for kk, vv in v.items():
            # print(f'{kk}: {vv}')
            result_map.update({
                kk: result_map.get(kk, 0) + 1
            })

    print(f'{"标签":10}\t{"标签个数":10} ')
    for label, counts in sorted(result_map.items(), key=lambda item: item[0]):
        print(f"{PyString.pad_string(label, 10)}\t{counts:10}")

    print(f'{"标签总数":10}\t{sum(result_map.values()):10}')
    return map_stat


def merge_json():
    # _dir_prefix = r'E:\Projects\pp-uiex-certs\anns\pred'
    _dir_prefix = r'E:\Projects\mmdet\data\anns\pred'
    PyFile.merge_json(
        list(filter(lambda x: 'pred_' in x, PyFile.get_file_list(_dir_prefix))),
        rf"{_dir_prefix}\pred_all.json",
        data={}
    )


def ls_UIEX_cert():
    # get_train_UIEX_cert_stats()
    # get_test_UIEX_cert_stats()

    # generate_UIEX_cert_train_data()
    # generate_UIEX_cert_test_data()

    # gen_UIEX_cert_tag_data()

    # merge_json()

    stat_UIEX_cert_acc()


if __name__ == '__main__':
    ls_UIEX_cert()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024-04-25 13:53
# @Author  : Jack
# @File    : PyShape

"""
PyShape
"""
from shapely import Polygon


def get_bounds(coords: list) -> tuple[float, float, float, float]:
    """
    计算多边形坐标点的最小边界框。

    Args:
        coords (list[tuple[float, float]]): 多边形坐标点列表，每个坐标点为(x, y)。

    Returns:
        tuple[float, float, float, float]: 最小边界框的左上角和右下角坐标点坐标，
            格式为(left, bottom, right, top)。

    """
    polygon = Polygon(coords)
    return polygon.bounds



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024-04-26 11:06
# @Author  : Jack
# @File    : PyPDF

"""
PyPDF
"""
from enum import Enum
from typing import List

import cv2
import numpy as np
from pdf2image import convert_from_path


class ImageFormat(Enum):
    """
    Image format 枚举
    """
    PIL = 'pil'
    OPENCV = 'opencv'


def convert_images(pdf_path: str, image_format: ImageFormat = ImageFormat.OPENCV) -> List[object]:
    """
    Convert pdf to images
    :param image_format: ImageFormat
    :param pdf_path:
    :return: List[Image.Image] or List[cv2.typing.MatLike]
    """
    images = convert_from_path(pdf_path)
    if image_format == ImageFormat.OPENCV:
        new_images = []
        for image in images:
            opencv_image = np.array(image)
            if opencv_image.shape[2] == 3:
                opencv_image = cv2.cvtColor(opencv_image, cv2.COLOR_RGB2BGR)
            new_images.append(opencv_image)
        return new_images
    return images



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024-05-13 15:48
# @Author  : Jack
# @File    : PyLabelStudio

"""
PyLabelStudio
"""
import json
import os

from NStudyPy import PyString, PyWeb


def convert_to_UIEX(file_paths: [str], main_dir='./data', domain=None, auth=None) -> None:
    """
    将LabelStudio的json文件转换为UIEX格式
    :param file_paths: 文件路径列表
    :param main_dir: 文件保存目录
    :param domain: 图片域名
    :param auth: 网站授权
    :return: None
    """
    skip_list = [
        '8_1693_1148_2e393926_286672_8a8087e78f135286018f9a26ca877240.jpg'
    ]
    _list = []
    for file_path in file_paths:
        with open(file_path, 'r', encoding='utf-8') as file:
            data = json.load(file)
            for item in data:
                project_id = item["project"]
                task_id = item["id"]
                data_image = item["data"].get("image")

                annotation = item["annotations"][0]
                ann_result = annotation["result"]
                if ann_result is None or len(ann_result) == 0 or sum(1 for x in ann_result if x['type'] == 'rectanglelabels') == 0:
                    continue
                task_completion_id = annotation["id"]
                new_image_name = f'{project_id}_{task_id}_{task_completion_id}_{str(item["file_upload"]).replace("-", "_")}'
                if new_image_name in skip_list:
                    continue
                image_path = os.path.join(main_dir, 'images', new_image_name)
                if not os.path.exists(image_path):
                    if domain is None:
                        raise Exception('domain or auth is None')
                    PyWeb.save_file_from_url(f'{domain}{data_image}', image_path,
                                             {
                                                 "Authorization": f"Token {auth}"
                                             })
                item["data"]["image"] = new_image_name
                _list.append(item)
    if len(_list) > 0:
        json_str = json.dumps(_list, indent=4, ensure_ascii=False)
        with open(os.path.join(main_dir, 'label_studio.json'), "w", encoding='utf8') as f:
            f.write(json_str)


def stat_UIEX_cert(file_paths: [str]) -> dict:
    """
    统计标签数量(有一个choices标签)
    :param file_paths: 文件路径列表
    :return: 标签数量字典
    """
    map_stat = {}
    for p in file_paths:
        with open(p, 'r', encoding='utf-8') as f:
            data = json.load(f)
            for item in data:
                # project_id = item["project"]
                # task_id = item["id"]

                result = item["annotations"][0]['result']
                choices = list(filter(lambda x: x['type'] == 'choices', result))
                if len(choices) == 0:
                    # print(f'project_id: {project_id} task_id:{task_id} this ann no choices')
                    continue
                choice_type = choices[0]['value']['choices'][0]

                choice_type_name = ''
                if choice_type == '1':
                    choice_type_name = '旧版本'
                elif choice_type == '2':
                    choice_type_name = '新版本'
                elif choice_type == '3':
                    choice_type_name = '电子版'

                for ann in result:
                    if ann['type'] == 'rectanglelabels':
                        label = ann['value']['rectanglelabels'][0]
                        choice_map = map_stat.get(label)
                        if choice_map is None:
                            map_stat.update({
                                label: {
                                    '旧版本': 0,
                                    '新版本': 0,
                                    '电子版': 0
                                }
                            })
                        choice_map = map_stat.get(label)
                        v = choice_map.get(choice_type_name)
                        choice_map.update({choice_type_name: v + 1})
                        map_stat.update({label: choice_map})

    print(f'{"类型":5}\t{"标签":10}\t{"标签个数":10} ')
    # 遍历并打印每个标签及其数量
    for t in ['电子版', '旧版本', '新版本']:
        for label, counts in map_stat.items():
            if t in counts:
                print(f"{t:<5}\t{PyString.pad_string(label, 10)}\t{counts[t]:10}")
    return map_stat


def get_new_image_name(item: dict) -> str:
    """
    获取新文件名
    :param item: label studio item
    :return: 新文件名
    """
    project_id = item["project"]
    task_id = item["id"]
    task_completion_id = item["annotations"][0]["id"]

    return f'{project_id}_{task_id}_{task_completion_id}_{str(item["file_upload"]).replace("-", "_")}'


def get_UIEX_cert_type(file_paths: [str]) -> dict:
    """
    获取标签类型(有一个choices标签)
    :param file_paths: 文件路径列表
    :return: 标签选择类型字典
    """
    map_stat = {}
    for p in file_paths:
        with open(p, 'r', encoding='utf-8') as f:
            data = json.load(f)
            for item in data:
                annotation = item["annotations"][0]
                ann_result = annotation["result"]
                if ann_result is None or len(ann_result) == 0 or sum(1 for x in ann_result if x['type'] == 'rectanglelabels') == 0:
                    continue
                new_image_name = get_new_image_name(item)

                choices = list(filter(lambda x: x['type'] == 'choices', ann_result))
                if len(choices) == 0:
                    print(f'project_id: {item["project"]} task_id:{ item["id"]} this ann no choices')
                    continue

                choice_type = choices[0]['value']['choices'][0]
                choice_type_name = ''
                if choice_type == '1':
                    choice_type_name = '旧版本'
                elif choice_type == '2':
                    choice_type_name = '新版本'
                elif choice_type == '3':
                    choice_type_name = '电子版'
                map_stat.update({
                    new_image_name: choice_type_name
                })
    return map_stat


def stat_UIEX_cert_acc(file_paths: [str]) -> dict:
    """
    统计标签准确率
    """
    map_stat = {}
    for p in file_paths:
        with open(p, 'r', encoding='utf-8') as f:
            data = json.load(f)
            for item in data:
                project_id = item["project"]
                task_id = item["id"]

                annotation = item["annotations"][0]
                task_completion_id = annotation["id"]

                ann_result = annotation["result"]
                if ann_result is None or len(ann_result) == 0 or sum(1 for x in ann_result if x['type'] == 'rectanglelabels') == 0:
                    continue
                new_image_name = f'{project_id}_{task_id}_{task_completion_id}_{str(item["file_upload"]).replace("-", "_")}'

                stat_value = {}
                _no_stat_value = {}
                map_label = {}
                for ann in ann_result:
                    if 'rectanglelabels' == ann['type']:
                        map_label.update({
                            ann['id']: ann['value']['rectanglelabels'][0]
                        })
                    elif 'textarea' == ann['type']:
                        _key = ann['id']
                        if _key in map_label:
                            stat_value.update({
                                map_label[_key]: ann['value']['text'][0]
                            })
                        else:
                            _no_stat_value.update({
                                _key: ann['value']['text'][0]
                            })
                for k, v in _no_stat_value.items():
                    stat_value.update({
                        map_label[k]: v
                    })
                map_stat.update({
                    new_image_name: stat_value
                })
    return map_stat


def get_real_points(ann):
    """
    get real points from annotation
    :param ann: label studio annotation
    :return: real points array
    """
    if 'original_width' not in ann or 'original_height' not in ann:
        return None
    w, h = ann['original_width'], ann['original_height']
    points = ann['value']['points']
    if points:
        return [[w * point[0] / 100.0, h * point[1] / 100.0] for point in points]
    return None


def convert_to_box(ann):
    """
    转换标注格式
    :param ann: 标注信息
    :return: [x, y, w, h]
    """
    if 'original_width' not in ann or 'original_height' not in ann:
        return None
    value = ann['value']
    w, h = ann['original_width'], ann['original_height']
    if all([key in value for key in ['x', 'y', 'width', 'height']]):
        x, y, w, h = w * value['x'] / 100.0, h * value['y'] / 100.0, w * value['width'] / 100.0, h * value['height'] / 100.0
        return [int(x), int(y), int(x + w), int(y + h)]
    return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024-04-28 14:13
# @Author  : Jack
# @File    : PyFile

"""
PyFile
"""
import os
import random
import shutil
import hashlib
import json


def check_file_ext(filename: str, allowed_extensions=['png', 'jpg', 'jpeg']) -> bool:
    """
    检查文件扩展名是否符合要求
    :param filename:
    :param allowed_extensions:
    :return:
    """
    return '.' in filename and filename.rsplit('.', 1)[1].lower() in allowed_extensions


def makedirs(path: str, is_file: bool = True):
    """
    创建目录
    :param is_file: 是否是文件
    :param path:
    :return:
    """
    if is_file:
        path = os.path.dirname(path)
    os.makedirs(path, exist_ok=True)


def get_filename(file_path: str) -> str:
    """
    获取文件名
    :param file_path: 文件路径
    :return: 文件名
    """
    return os.path.basename(file_path)


def get_directory(file_path: str) -> str:
    """
    获取文件所在目录
    :param file_path: 文件路径 (也可能是目录, 如果是目录返回上级目录)
    :return: 目录路径
    """
    return os.path.dirname(file_path)


def random_split_s(source_dir: str, target_dir: str, s=(100, 400, 250, 250)) -> None:
    """
    随机拆分文件; 不处理子文件夹
    :param source_dir: 源文件夹
    :param target_dir: 目标文件夹
    :param s: 拆分个数 (100, 400, 250, 250) 是每个分组的个数，直至所有文件都拆分完毕
    :return: None
    """
    files = os.listdir(source_dir)
    random.shuffle(files)

    sum_s = 0
    for idx, count in enumerate(s):
        _no = f'{idx:03}'
        image_g_dir = os.path.join(target_dir, _no)
        if not os.path.exists(image_g_dir):
            os.makedirs(image_g_dir)
        for file_name in files[sum_s:sum_s + count]:
            shutil.copy(os.path.join(source_dir, file_name), os.path.join(image_g_dir, file_name))
        sum_s += count
        if sum_s >= len(files):
            break
    # Done it


def random_spilt(source_dir: str, target_dir: str, spilt_count=200) -> None:
    """
    把一个文件夹的文件,随机分成 spilt_count 一个文件夹 ; 不处理子文件夹
    :param source_dir: 源文件夹
    :param target_dir: 目标文件夹
    :param spilt_count: 每个文件夹的文件数量
    :return: None
    """
    files = os.listdir(source_dir)

    random.shuffle(files)

    # group_files_len = (len(files) + spilt_count - 1) // spilt_count

    group_files = [files[i:i + spilt_count] for i in range(0, len(files), spilt_count)]
    for idx, g in enumerate(group_files):
        _no = f'{idx:03}'
        image_g_dir = os.path.join(target_dir, _no)
        if not os.path.exists(image_g_dir):
            os.makedirs(image_g_dir)
        for i in g:
            shutil.copy(os.path.join(source_dir, i), os.path.join(image_g_dir, i))


def get_file_list(path: str, is_recursive=True) -> list:
    """
    获取文件列表
    :param path: 路径
    :param is_recursive: 是否递归
    :return: 文件列表
    """
    if not os.path.exists(path):
        raise FileNotFoundError('Path does not exist')
    if os.path.isdir(path):
        files = []
        for root, _, f_names in os.walk(path):
            for f_name in f_names:
                files.append(os.path.join(root, f_name))
            if not is_recursive:
                break
    else:
        files = [path]
    return files


def get_md5(file_path) -> str:
    """
    获取文件的md5值
    :param file_path: 文件路径
    :return: md5值
    """
    if not os.path.exists(file_path):
        raise FileNotFoundError('Path does not exist')
    hash_md5 = hashlib.md5()
    with open(file_path, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()


def get_repeat_file(path: str, is_recursive=True) -> dict:
    """
    获取重复文件
    :param path: 路径
    :param is_recursive:  是否递归
    :return: dict {"md5" : [file1,file2]}
    """
    file_dict = dict()
    for f in get_file_list(path, is_recursive):
        md5 = get_md5(f)
        if md5 not in file_dict:
            file_dict[md5] = [f]
        else:
            file_dict[md5].append(f)
    repeat_files = dict()
    for k, v in file_dict.items():
        if len(v) > 1:
            repeat_files.update({k: v})
            print(f'{k} {len(v)}')
    # return list(filter(lambda x: len(x) > 1, file_dict.values()))
    return repeat_files


def delete_repeat_file(path: str, is_recursive=True) -> None:
    """
    删除重复文件
    :param path: 路径
    :param is_recursive:  是否递归
    :return: None
    """
    for f in get_repeat_file(path, is_recursive).values():
        for i in f[1:]:
            os.remove(i)


def save_json(data: object, path: str, ) -> None:
    """
    保存json文件
    :param data: 数据
    :param path: 路径
    :return: None
    """
    makedirs(path, is_file=True)
    with open(path, 'w', encoding='utf-8') as f:
        f.write(json.dumps(data, ensure_ascii=False, indent=4))


def read_json(path: str) -> object:
    """
    获取json文件
    :param path: 路径
    :return: 数据
    """
    with open(path, 'r', encoding='utf-8') as f:
        data = json.loads(f.read())
    return data


def merge_json(source_files: [str], target_file: str, data={}) -> None:
    """
    合并json文件,支持字典类型和列表类型
    :param source_files: 源文件列表
    :param target_file: 目标文件
    :param data: 初始化数据类型
    :return: None
    """
    for f in source_files:
        with open(f, 'r', encoding='utf-8') as f1:
            if type(data) is dict:
                data = {**data, **json.loads(f1.read())}
            elif type(data) is list:
                data.extend(json.loads(f1.read()))
    save_json(data, target_file)


def remove_file(path):
    """
    删除指定路径的文件或目录。
    :param path: 要删除的文件或目录的路径。
    """
    try:
        if os.path.isfile(path) or os.path.islink(path):
            os.remove(path)
        elif os.path.isdir(path):
            shutil.rmtree(path)
    except Exception:
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024-06-04 15:18
# @Author  : Jack
# @File    : PyEnv

"""
PyEnv
"""

import os
import json


def update_home_env(config_file: str) -> None:
    """
        update home environment variables
    """
    config_path = os.path.join(os.path.expanduser("~"), '.home', config_file)
    if os.path.exists(config_path):
        with open(config_path, 'r', encoding='utf-8') as f:
            config = json.load(f)
            for key, value in config.items():
                os.environ[key] = value


def get_env(key: str, default=None, value_type=None):
    """
        Returns the value from the key.
        check environment variables
    """
    value = default

    if key in os.environ:
        value = os.environ.get(key)

    if (value_type is None) or (default is None):
        return value

    if value_type == bool:
        return value == "True"

    return value_type(value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024-04-25 13:39
# @Author  : Jack
# @File    : PyString

"""
PyString
"""
import random
import string

from wcwidth import wcswidth

from NStudyPy import PyDate


def get_random_id(length: int = 10) -> str:
    """
    生成指定长度的随机字符串

    Args:
        length (int, optional): 字符串长度，默认为10.

    Returns:
        str: 生成的随机字符串.
    """
    return ''.join(random.choices(string.ascii_letters + string.digits, k=length))


def get_trace_id(length: int = 5) -> str:
    """
    生成指定长度的随机字符串，并加上时间戳
    Args:
        length (int, optional): 字符串长度，默认为5.
    Returns:
        str: 生成的随机字符串.
    """
    return f'{PyDate.get_date()}_{get_random_id(length)}'


def pad_string(s, width) -> str:
    """
    填充字符串到指定宽度
    :param s: 字符串
    :param width: 指定宽度
    :return: 填充后的字符串
    """
    return s + ' ' * (width - wcswidth(s))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024-06-04 16:09
# @Author  : Jack
# @File    : test_PyFile

"""
test_PyFile
"""
import json

from NStudyPy import PyFile


def test_random_split_s():
    tag = '职称证'
    PyFile.random_split_s(source_dir=rf'F:\temp\cards\{tag}', target_dir=rf'F:\temp\target\{tag}')


def test_convert_json():
    file_name = r'pred_3.json'
    map_json = PyFile.read_json(file_name)
    new_map = {}
    for key, value in map_json.items():
        try:
            new_map[key] = json.loads(value)
        except Exception as e:
            print(f"An unexpected error occurred: {e}")
    PyFile.save_json(new_map, file_name)


if __name__ == '__main__':
    # PyFile.delete_repeat_file(r'F:\temp\cards\xxxx')
    # test_random_split_s
    # test_convert_json()
    print(PyFile.get_filename(r'F:\temp\cards\xxxx\user.txt'))
    print(PyFile.get_directory(r'F:\temp\cards\xxxx'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024-06-28 18:20
# @Author  : Jack
# @File    : test_LabelStudio_mmdet

"""
test_LabelStudio_mmdet
"""
import shutil
import time
import random
import os
import cv2
from NStudyPy import PyFile


def merge_coco_data():
    result = {
        "images": [],
        "categories": [],
        "annotations": [],
        "info": {
            "year": 2024,
            "version": "1.0",
            "description": "code render",
            "contributor": "code render",
            "url": "",
            "date_created": time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        }
    }

    json_files = [
        r'E:\Projects\mmdet\ls_source\11\result.json',
        r'E:\Projects\mmdet\ls_source\12\result.json',
        r'E:\Projects\mmdet\ls_source\13\result.json'
    ]
    ann_id = 0
    img_id = 10000
    for idx, file_path in enumerate(json_files):
        data = PyFile.read_json(file_path)
        if idx == 1:
            result['categories'] = list(map(lambda x: {'id': x['id'] - 3, 'name': x['name']}, data['categories'][3:]))
            print(list(map(lambda x: x['name'], result['categories'])))
        images = data['images']
        annotations = data['annotations']

        img_id_convert = {}
        for img in images:
            file_name = img['file_name']
            target_file_path = os.path.join(r'E:\Projects\mmdet\data\data', file_name)
            if not os.path.exists(target_file_path):
                shutil.copyfile(os.path.join(PyFile.get_directory(file_path), file_name), target_file_path)

            img_id_convert[img['id']] = img_id
            img['id'] = img_id

            img_id = img_id + 1

        result['images'].extend(images)

        for ann in annotations:
            ann['id'] = ann_id
            ann['image_id'] = img_id_convert[ann['image_id']]
            ann['category_id'] = ann['category_id'] - 3

            ann_id = ann_id + 1

        result['annotations'].extend(annotations)

    PyFile.save_json(result, r'E:\Projects\mmdet\data\data\result.json')


def spilt_data(spilt=(0.95, 0.05)):
    data = PyFile.read_json(r'E:\Projects\mmdet\data\data\result.json')
    images = data['images']
    annotations = data['annotations']
    categories = data['categories']
    info = data['info']

    random.shuffle(images)
    train_images = images[:int(len(images) * spilt[0])]
    val_images = images[len(train_images):]
    print(len(images), len(train_images), len(val_images))

    train_result = {
        "images": train_images,
        "categories": categories,
        "annotations": [],
        "info": info
    }
    val_result = {
        "images": val_images,
        "categories": categories,
        "annotations": [],
        "info": info
    }
    for ann in annotations:
        if ann['image_id'] in map(lambda x: x['id'], train_images):
            train_result['annotations'].append(ann)
        else:
            val_result['annotations'].append(ann)
    PyFile.save_json(train_result, r'E:\Projects\mmdet\data\data\train.json')
    PyFile.save_json(val_result, r'E:\Projects\mmdet\data\data\val.json')


def show_data():
    image_path = r'E:\Projects\mmdet\data\data'
    result = PyFile.read_json(rf'{image_path}\result.json')
    categories = result['categories']
    annotations = result['annotations']
    images = result['images']

    for image in images[:2]:
        img = cv2.imread(os.path.join(image_path, image['file_name']))

        anns = list(filter(lambda x: x['image_id'] == image['id'], annotations))
        for ann in anns:
            x1, y1, w, h = ann['bbox']
            x2 = x1 + w
            y2 = y1 + h
            cv2.rectangle(img, (int(x1), int(y1)), (int(x2), int(y2)), (0, 0, 255))

            cat_id = ann['category_id']
            cat_name = list(filter(lambda x: x['id'] == cat_id, categories))[0]['name']
            cv2.putText(img, str(cat_id), (int(x1), int(y1)), cv2.FONT_HERSHEY_SIMPLEX, 0.5, (0, 255, 0))

        cv2.imshow(image['file_name'], img)

        cv2.waitKey(0)
        cv2.destroyAllWindows()


if __name__ == '__main__':
    # merge_coco_data()

    spilt_data()

    # show_data()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024-07-02 12:00
# @Author  : Jack
# @File    : test_LabelStudio_mmdet_ex

"""
test_LabelStudio_mmdet_ex
"""

import os
import shutil
import time

from NStudyPy import PyFile


def merge_coco_data_all():
    result = {
        "images": [],
        "categories": [],
        "annotations": [],
        "info": {
            "year": 2024,
            "version": "1.0",
            "description": "code render",
            "contributor": "code render",
            "url": "",
            "date_created": time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        }
    }

    json_files = [
        r'E:\Projects\mmdet\data\data_1\result.json',
        r'E:\Projects\mmdet\data\data_2\result.json',
        r'E:\Projects\mmdet\data\data_3\result.json'
    ]

    category_id = 0
    ann_id = 0
    img_id = 0
    for file_path in json_files:
        data = PyFile.read_json(file_path)

        category_id_convert = {}
        categories = []
        for category in data['categories']:
            category_find = next(filter(lambda x: x['name'] == category['name'], result['categories']), None)
            if category_find is None:
                category_id_convert[category['id']] = category_id
                category['id'] = category_id
                categories.append(category)
                category_id = category_id + 1
            else:
                category_id_convert[category['id']] = category_find['id']

        result['categories'].extend(categories)

        images = data['images']
        img_id_convert = {}
        for img in data['images']:
            img_id_convert[img['id']] = img_id
            img['id'] = img_id

            img_id = img_id + 1
        result['images'].extend(images)

        annotations = data['annotations']
        for ann in annotations:
            ann['id'] = ann_id
            ann['image_id'] = img_id_convert[ann['image_id']]
            ann['category_id'] = category_id_convert[ann['category_id']]

            ann_id = ann_id + 1
        result['annotations'].extend(annotations)
    print(list(map(lambda x: x['name'], result['categories'])))
    PyFile.save_json(result, r'E:\Projects\mmdet\data\data\result.json')


if __name__ == '__main__':
    merge_coco_data_all()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024-07-01 17:34
# @Author  : Jack
# @File    : test_PyDate

"""
test_PyDate
"""

from NStudyPy import PyDate

# 示例日期字符串
date_strs = [
    '有效2024-06-18', "日期:2024-06-18结束", '2024-06-18有效',
    '日期:2024/06/18', '2024/6/18', '2024年06月18日',
    '日期2024年6月8日', '24年6月8日', '024年6月8日',
]

for d in date_strs:
    print(f"原始日期字符串：{d}\t\t 格式化后字符串:\t\t{PyDate.format_date(d)}")
